# Copyright 2016 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Common tags used for graphs in SavedModel.

"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from tensorflow.python.util.all_util import remove_undocumented


# Tag for the `serving` graph.
SERVING = "serve"

# Tag for the `training` graph.
TRAINING = "train"

# Tag for the `gpu` graph.
GPU = "gpu"

# Tag for the `tpu` graph.
TPU = "tpu"

_allowed_symbols = [
    "SERVING",
    "TRAINING",
    "GPU",
    "TPU"
]
remove_undocumented(__name__, _allowed_symbols)
